@@ -49,7 +49,7 @@ namespace QuattroSpreadsheetInternal
 struct Style : public WPSCellFormat
 {
 	//! construtor
-	Style(libwps_tools_win::Font::Type type) : WPSCellFormat(), m_fontType(type), m_fileFormat(0xFF), m_extra("")
+	explicit Style(libwps_tools_win::Font::Type type) : WPSCellFormat(), m_fontType(type), m_fileFormat(0xFF), m_extra("")
 	{
 	}
 
@@ -139,7 +139,7 @@ class Cell : public WPSCell
 {
 public:
 	/// constructor
-	Cell(libwps_tools_win::Font::Type type) : m_fontType(type), m_fileFormat(0xFF), m_content() { }
+	explicit Cell(libwps_tools_win::Font::Type type) : m_fontType(type), m_fileFormat(0xFF), m_content() { }
 
 	//! operator<<
 	friend std::ostream &operator<<(std::ostream &o, Cell const &cell);
@@ -277,30 +277,13 @@ std::ostream &operator<<(std::ostream &o, Cell const &cell)
 //! the spreadsheet of a WPS4Spreadsheet
 class Spreadsheet
 {
-protected:
-	//! a comparaison structure used to sort cell by rows and and columns
-	struct ComparePosition
-	{
-		//! constructor
-		ComparePosition() {}
-		//! comparaison function
-		bool operator()(Vec2i const &c1, Vec2i const &c2) const
-		{
-			if (c1[1]!=c2[1])
-				return c1[1]<c2[1];
-			return c1[0]<c2[0];
-		}
-	};
-
 public:
 	//! the spreadsheet type
 	enum Type { T_Spreadsheet, T_Filter, T_Report };
 
-	typedef std::map<Vec2i, Cell, ComparePosition> PositionToCellMap_t;
-
 	//! a constructor
-	Spreadsheet(Type type=T_Spreadsheet, int id=0) : m_type(type), m_id(id), m_numCols(0), m_numRows(0), m_LBPosition(-1,-1),
-		m_widthCols(), m_heightRows(), m_positionToCellMap(), m_lastCellPos(),
+	Spreadsheet(Type type=T_Spreadsheet, int id=0) : m_type(type), m_id(id), m_numCols(0), m_numRows(0),
+		m_widthCols(), m_rowHeightMap(), m_heightDefault(16), m_positionToCellMap(), m_lastCellPos(),
 		m_rowPageBreaksList() {}
 	//! return a cell corresponding to a spreadsheet, create one if needed
 	Cell &getCell(Vec2i const &pos, libwps_tools_win::Font::Type type)
@@ -309,8 +292,7 @@ public:
 		{
 			Cell cell(type);
 			cell.setPosition(pos);
-			m_positionToCellMap.insert
-			(std::map<Vec2i, Cell, ComparePosition>::value_type(pos,cell));
+			m_positionToCellMap.insert(std::map<Vec2i, Cell>::value_type(pos,cell));
 		}
 		m_lastCellPos=pos;
 		return m_positionToCellMap.find(pos)->second;
@@ -330,16 +312,8 @@ public:
 		m_widthCols[size_t(col)] = w;
 		if (col >= m_numCols) m_numCols=col+1;
 	}
-	//! set the rows size
-	void setRowHeight(int row, int h=-1)
-	{
-		if (row < 0) return;
-		if (row >= int(m_heightRows.size())) m_heightRows.resize(size_t(row)+1, -1);
-		m_heightRows[size_t(row)] = h;
-		if (row >= m_numRows) m_numRows=row+1;
-	}
 
-	//! convert the m_widthCols, m_heightRows in a vector of of point size
+	//! convert the m_widthCols in a vector of of point size
 	static std::vector<float> convertInPoint(std::vector<int> const &list,
 	                                         float defSize)
 	{
@@ -353,17 +327,67 @@ public:
 		}
 		return res;
 	}
-	/** compute the last Right Bottom cell position */
-	void computeRightBottomPosition()
+	//! returns the row size in point
+	float getRowHeight(int row) const
 	{
-		int maxX = -1, maxY = -1;
-		for (PositionToCellMap_t::const_iterator it=m_positionToCellMap.begin(); it!=m_positionToCellMap.end(); ++it)
+		std::map<Vec2i,int>::const_iterator rIt=m_rowHeightMap.lower_bound(Vec2i(-1,row));
+		if (rIt!=m_rowHeightMap.end() && rIt->first[0]<=row && rIt->first[1]>=row)
+			return (float) rIt->second;
+		return (float) m_heightDefault;
+	}
+	//! returns the height of a row in point and updated repeated row
+	float getRowHeight(int row, int &numRepeated) const
+	{
+		std::map<Vec2i,int>::const_iterator rIt=m_rowHeightMap.lower_bound(Vec2i(-1,row));
+		if (rIt!=m_rowHeightMap.end() && rIt->first[0]<=row && rIt->first[1]>=row)
 		{
-			Vec2i const &p = it->second.position();
-			if (p[0] > maxX) maxX = p[0];
-			if (p[1] > maxY) maxY = p[1];
+			numRepeated=rIt->first[1]-row+1;
+			return (float) rIt->second;
 		}
-		m_LBPosition=Vec2i(maxX, maxY);
+		numRepeated=10000;
+		return (float) m_heightDefault;
+	}
+	//! set the rows size
+	void setRowHeight(int row, int h)
+	{
+		if (h>=0)
+			m_rowHeightMap[Vec2i(row,row)]=h;
+	}
+	//! try to compress the list of row height
+	void compressRowHeights()
+	{
+		std::map<Vec2i,int> oldMap=m_rowHeightMap;
+		m_rowHeightMap.clear();
+		std::map<Vec2i,int>::const_iterator rIt=oldMap.begin();
+		int actHeight=-1;
+		Vec2i actPos(0,-1);
+		while (rIt!=oldMap.end())
+		{
+			// first check for not filled row
+			if (rIt->first[0]!=actPos[1]+1)
+			{
+				if (actHeight==m_heightDefault)
+					actPos[1]=rIt->first[0]-1;
+				else
+				{
+					if (actPos[1]>=actPos[0])
+						m_rowHeightMap[actPos]=actHeight;
+					actHeight=m_heightDefault;
+					actPos=Vec2i(actPos[1]+1, rIt->first[0]-1);
+				}
+			}
+			if (rIt->second!=actHeight)
+			{
+				if (actPos[1]>=actPos[0])
+					m_rowHeightMap[actPos]=actHeight;
+				actPos[0]=rIt->first[0];
+				actHeight=rIt->second;
+			}
+			actPos[1]=rIt->first[1];
+			++rIt;
+		}
+		if (actPos[1]>=actPos[0])
+			m_rowHeightMap[actPos]=actHeight;
 	}
 	//! returns true if the spreedsheet is empty
 	bool empty() const
@@ -378,15 +402,15 @@ public:
 	int m_numCols;
 	/** the number of rows */
 	int m_numRows;
-	/** the final Right Bottom position, computed by updateState */
-	Vec2i m_LBPosition;
 
 	/** the column size in TWIP (?) */
 	std::vector<int> m_widthCols;
-	/** the row size in TWIP (?) */
-	std::vector<int> m_heightRows;
+	/** the map Vec2i(min row, max row) to size in points */
+	std::map<Vec2i,int> m_rowHeightMap;
+	/** the default row size in point */
+	int m_heightDefault;
 	/** a map cell to not empty cells */
-	PositionToCellMap_t m_positionToCellMap;
+	std::map<Vec2i, Cell> m_positionToCellMap;
 	/** the last cell position */
 	Vec2i m_lastCellPos;
 	/** the list of row page break */
@@ -559,7 +583,6 @@ bool QuattroSpreadsheet::readSheetSize()
 	if (nRow==-1 && nCol==0) return true;
 	if (nRow < 0 || nCol <= 0) return false;
 
-	m_state->getActualSheet().setRowHeight(nRow-1);
 	m_state->getActualSheet().setColumnWidth(nCol-1);
 	return true;
 
@@ -585,25 +608,13 @@ bool QuattroSpreadsheet::readRowSize()
 	int row = libwps::read16(m_input);
 	int height = libwps::readU8(m_input);
 
-	bool ok = row >= 0 && row < m_state->getActualSheet().m_numRows+20;
+	bool ok = row >= 0;
 	f << "Entries(Row):Row" << row << "";
 	if (!ok) f << "###";
 	f << ":height=" << height << ",";
 
 	if (ok)
-	{
-		if (row >= m_state->getActualSheet().m_numRows)
-		{
-			static bool first = true;
-			if (first)
-			{
-				first = false;
-				WPS_DEBUG_MSG(("QuattroSpreadsheet::readRowSize: I must increase the number of rows\n"));
-			}
-			f << "#row[inc],";
-		}
-		m_state->getActualSheet().setRowHeight(row, 20*height); // point to TWIP
-	}
+		m_state->getActualSheet().setRowHeight(row, height); // point to TWIP
 	ascii().addPos(pos);
 	ascii().addNote(f.str().c_str());
 
@@ -1668,7 +1679,7 @@ bool QuattroSpreadsheet::readCell
 			else
 			{
 				val &= 0x3FFF;
-				if (val & 0x2000) val = val - 0x4000;
+				if (val>0x1000) val = val - 0x2000;
 			}
 			if (dim==2)
 				val += sheetId;
@@ -2146,23 +2157,37 @@ void QuattroSpreadsheet::sendSpreadsheet(int sId)
 		}
 		sheet.reset(new QuattroSpreadsheetInternal::Spreadsheet);
 	}
-	sheet->computeRightBottomPosition();
 
 	m_listener->openSheet(sheet->convertInPoint(sheet->m_widthCols,76), librevenge::RVNG_POINT,
-	                      m_state->getSheetName(sId));
-	std::vector<float> rowHeight = sheet->convertInPoint(sheet->m_heightRows,16);
-	for (int r=0; r<=sheet->m_LBPosition[1]; ++r)
+	                      std::vector<int>(), m_state->getSheetName(sId));
+	sheet->compressRowHeights();
+	std::map<Vec2i, QuattroSpreadsheetInternal::Cell>::const_iterator it = sheet->m_positionToCellMap.begin();
+	int prevRow = -1;
+	while (it != sheet->m_positionToCellMap.end())
 	{
-		m_listener->openSheetRow(r < int(rowHeight.size()) ? rowHeight[size_t(r)] : 16, librevenge::RVNG_POINT);
-		for (int c=0; c<=sheet->m_LBPosition[0]; ++c)
+		int row=it->first[1];
+		QuattroSpreadsheetInternal::Cell const &cell=(it++)->second;
+		if (row>prevRow+1)
 		{
-			if (sheet->m_positionToCellMap.find(Vec2i(c,r))==sheet->m_positionToCellMap.end())
-				continue;
-			QuattroSpreadsheetInternal::Cell const &cell= sheet->m_positionToCellMap.find(Vec2i(c,r))->second;
-			sendCellContent(cell);
+			while (row > prevRow+1)
+			{
+				if (prevRow != -1) m_listener->closeSheetRow();
+				int numRepeat;
+				float h=sheet->getRowHeight(prevRow+1, numRepeat);
+				if (row<prevRow+1+numRepeat)
+					numRepeat=row-1-prevRow;
+				m_listener->openSheetRow(h, librevenge::RVNG_POINT, false, numRepeat);
+				prevRow+=numRepeat;
+			}
 		}
-		m_listener->closeSheetRow();
+		if (row!=prevRow)
+		{
+			if (prevRow != -1) m_listener->closeSheetRow();
+			m_listener->openSheetRow(sheet->getRowHeight(++prevRow), librevenge::RVNG_POINT);
+		}
+		sendCellContent(cell);
 	}
+	if (prevRow!=-1) m_listener->closeSheetRow();
 	m_listener->closeSheet();
 }
 
@@ -2174,7 +2199,6 @@ void QuattroSpreadsheet::sendCellContent(QuattroSpreadsheetInternal::Cell const
 		return;
 	}
 
-	librevenge::RVNGPropertyList propList;
 	libwps_tools_win::Font::Type fontType = cell.m_fontType;
 	m_listener->setFont(cell.getFont());
 
@@ -2190,16 +2214,16 @@ void QuattroSpreadsheet::sendCellContent(QuattroSpreadsheetInternal::Cell const
 		for (size_t c=0; c < text.length(); ++c)
 		{
 			if (!hasLICS)
-				WPSListener::appendUnicode
+				libwps::appendUnicode
 				((uint32_t)libwps_tools_win::Font::unicode((unsigned char)text[c],fontType), finalString);
 			else
-				WPSListener::appendUnicode
+				libwps::appendUnicode
 				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c],fontType), finalString);
 		}
 		text=finalString.cstr();
 	}
 	finalCell.updateFormat();
-	m_listener->openSheetCell(finalCell, content, propList);
+	m_listener->openSheetCell(finalCell, content);
 
 	if (cell.m_content.m_textEntry.valid())
 	{
