@@ -52,7 +52,7 @@ namespace WKS4ParserInternal
 struct Font : public WPSFont
 {
 	//! constructor
-	Font() : WPSFont(), m_type(libwps_tools_win::Font::DOS_850)
+	explicit Font(libwps_tools_win::Font::Type type) : WPSFont(), m_type(type)
 	{
 	}
 	//! font encoding type
@@ -112,12 +112,24 @@ void SubDocument::parse(shared_ptr<WKSContentListener> &listener, libwps::SubDoc
 struct State
 {
 	//! constructor
-	State() :  m_eof(-1), m_isSpreadsheet(true), m_version(-1), m_fontsList(), m_pageSpan(), m_actPage(0), m_numPages(0),
+	explicit State(libwps_tools_win::Font::Type fontType) :
+		m_eof(-1), m_creator(libwps::WPS_MSWORKS), m_isSpreadsheet(true), m_fontType(fontType), m_version(-1),
+		m_hasLICSCharacters(false), m_fontsList(), m_pageSpan(), m_actPage(0), m_numPages(0),
 		m_headerString(""), m_footerString("")
 	{
 	}
 	//! returns a color corresponding to an id
-	bool getColor(int id, uint32_t &color) const;
+	bool getColor(int id, WPSColor &color) const;
+	//! return the default font style
+	libwps_tools_win::Font::Type getDefaultFontType() const
+	{
+		if (m_fontType != libwps_tools_win::Font::UNKNOWN)
+			return m_fontType;
+		if (m_version>2)
+			return libwps_tools_win::Font::WIN3_WEUROPE;
+		return m_creator==libwps::WPS_MSWORKS ? libwps_tools_win::Font::DOS_850 : libwps_tools_win::Font::CP_437;
+	}
+
 	//! returns a default font (Courier12) with file's version to define the default encoding */
 	WPSFont getDefaultFont() const
 	{
@@ -132,10 +144,16 @@ struct State
 
 	//! the last file position
 	long m_eof;
+	//! the application
+	libwps::WPSCreator m_creator;
 	//! boolean to know if the file is a spreadsheet file or a database file
 	bool m_isSpreadsheet;
+	//! the user font type
+	libwps_tools_win::Font::Type m_fontType;
 	//! the file version
 	int m_version;
+	//! flag to know if the character
+	bool m_hasLICSCharacters;
 	//! the fonts list
 	std::vector<Font> m_fontsList;
 	//! the actual document size
@@ -147,7 +165,7 @@ struct State
 	std::string m_footerString;
 };
 
-bool State::getColor(int id, uint32_t &color) const
+bool State::getColor(int id, WPSColor &color) const
 {
 	if (m_version<=2)
 	{
@@ -161,7 +179,7 @@ bool State::getColor(int id, uint32_t &color) const
 			WPS_DEBUG_MSG(("WKS4ParserInternal::State::getColor(): unknown Dos color id: %d\n",id));
 			return false;
 		}
-		color=colorDosMap[id];
+		color=WPSColor(colorDosMap[id]);
 		return true;
 	}
 	static const uint32_t colorMap[]=
@@ -179,18 +197,19 @@ bool State::getColor(int id, uint32_t &color) const
 		WPS_DEBUG_MSG(("WKS4ParserInternal::State::getColor(): unknown color id: %d\n",id));
 		return false;
 	}
-	color = colorMap[id];
+	color = WPSColor(colorMap[id]);
 	return true;
 }
 
 }
 
 // constructor, destructor
-WKS4Parser::WKS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header) :
+WKS4Parser::WKS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header,
+                       libwps_tools_win::Font::Type encoding) :
 	WKSParser(input, header), m_listener(), m_state(), m_spreadsheetParser()
 
 {
-	m_state.reset(new WKS4ParserInternal::State);
+	m_state.reset(new WKS4ParserInternal::State(encoding));
 	m_spreadsheetParser.reset(new WKS4Spreadsheet(*this));
 }
 
@@ -216,10 +235,20 @@ bool WKS4Parser::checkFilePosition(long pos)
 	return pos <= m_state->m_eof;
 }
 
+libwps_tools_win::Font::Type WKS4Parser::getDefaultFontType() const
+{
+	return m_state->getDefaultFontType();
+}
+
+libwps::WPSCreator WKS4Parser::getCreator() const
+{
+	return m_state->m_creator;
+}
+
 //////////////////////////////////////////////////////////////////////
 // interface with WKS4Spreadsheet
 //////////////////////////////////////////////////////////////////////
-bool WKS4Parser::getColor(int id, uint32_t &color) const
+bool WKS4Parser::getColor(int id, WPSColor &color) const
 {
 	return m_state->getColor(id, color);
 }
@@ -237,6 +266,13 @@ bool WKS4Parser::getFont(int id, WPSFont &font, libwps_tools_win::Font::Type &ty
 	return true;
 }
 
+bool WKS4Parser::hasLICSCharacters() const
+{
+	if (version()<=2)
+		return m_state->m_hasLICSCharacters;
+	return false;
+}
+
 // main function to parse the document
 void WKS4Parser::parse(librevenge::RVNGSpreadsheetInterface *documentInterface)
 {
@@ -262,7 +298,10 @@ void WKS4Parser::parse(librevenge::RVNGSpreadsheetInterface *documentInterface)
 			m_spreadsheetParser->setListener(m_listener);
 
 			m_listener->startDocument();
-			m_spreadsheetParser->sendSpreadsheet();
+			int numSheet=m_spreadsheetParser->getNumSpreadsheets();
+			if (numSheet==0) ++numSheet;
+			for (int i=0; i<numSheet; ++i)
+				m_spreadsheetParser->sendSpreadsheet(i);
 			m_listener->endDocument();
 			m_listener.reset();
 			ok = true;
@@ -306,7 +345,7 @@ shared_ptr<WKSContentListener> WKS4Parser::createListener(librevenge::RVNGSpread
 ////////////////////////////////////////////////////////////
 bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 {
-	*m_state = WKS4ParserInternal::State();
+	*m_state = WKS4ParserInternal::State(m_state->m_fontType);
 	libwps::DebugStream f;
 
 	RVNGInputStreamPtr input = getInput();
@@ -319,26 +358,32 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 	input->seek(0,librevenge::RVNG_SEEK_SET);
 	int firstOffset = (int) libwps::readU8(input);
 	int type = (int) libwps::read8(input);
+	bool needEncoding=true;
 	f << "FileHeader:";
 	if ((firstOffset == 0 && type == 0) ||
 	        (firstOffset == 0x20 && type == 0x54))
 	{
-		m_state->m_version=2;
+		m_state->m_version=1;
 		f << "DOS,";
 	}
 	else if (firstOffset == 0xff)
 	{
 		f << "Windows,";
 		m_state->m_version=3;
+		needEncoding=false;
 	}
 	else
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find unexpected first data\n"));
 		return false;
 	}
+	libwps::WPSCreator creator=libwps::WPS_MSWORKS;
+	libwps::WPSKind kind=libwps::WPS_SPREADSHEET;
+	bool isSpreadsheet=true;
 	if (type == 0x54)
 	{
-		m_state->m_isSpreadsheet=false;
+		isSpreadsheet=false;
+		kind=libwps::WPS_DATABASE;
 		f << "database,";
 	}
 	else if (type == 0)
@@ -348,31 +393,84 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 		WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find unexpected type file\n"));
 		return false;
 	}
-	if (libwps::read16(input) != 2)
+	int val=(int) libwps::read16(input);
+	if (val==2)
+	{
+		// version
+		val=(int) libwps::readU16(input);
+		if (isSpreadsheet)
+		{
+			if (val==0x404)
+			{
+			}
+			else if (val==0x405)
+			{
+				f << "symphony,";
+				creator=libwps::WPS_SYMPHONY;
+			}
+			else if (val==0x406)
+			{
+				m_state->m_version=1;
+				f << "lotus,";
+				creator=libwps::WPS_LOTUS;
+			}
+			else if (val==0x5120 || val==0x5121)
+			{
+				WPS_DEBUG_MSG(("WKS4Parser::checkHeader: must not be called with a DOS Quattro file\n"));
+				return false;
+			}
+			else if (val==0x8006)
+			{
+				WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find lotus file format, sorry parsing this format is not implemented\n"));
+				return false;
+			}
+#ifdef DEBUG
+			else if (val==0x1002)
+			{
+				WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find a quattro pro windows file, sorry parsing this format is not implemented\n"));
+				m_state->m_version=1000;
+				f << "quattropro,";
+				creator=libwps::WPS_QUATTRO_PRO;
+			}
+#endif
+			else
+			{
+#ifdef DEBUG
+				f << "vers=" << std::hex << val << std::dec << ",";
+#else
+				WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find unknown file version\n"));
+				return false;
+#endif
+			}
+		}
+		else if (val)
+			return false;
+	}
+	else
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::checkHeader: header contain unexpected size field data\n"));
 		return false;
 	}
-	if ((m_state->m_isSpreadsheet && libwps::read16(input) != 0x0404) ||
-	        (!m_state->m_isSpreadsheet && libwps::read16(input) != 0))
+
+	input->seek(0, librevenge::RVNG_SEEK_SET);
+	if (strict && m_state->m_version<1000)
 	{
-		WPS_DEBUG_MSG(("WKS4Parser::checkHeader: header contain unknown data\n"));
-		return false;
-	}
-	if (strict)
-	{
-		for (int i=0; i < 3; ++i)
+		for (int i=0; i < 4; ++i)
 		{
 			if (!readZone()) return false;
 		}
 	}
 	ascii().addPos(0);
 	ascii().addNote(f.str().c_str());
-	ascii().addPos(6);
+
+	m_state->m_isSpreadsheet=isSpreadsheet;
+	m_state->m_creator=creator;
 	if (header)
 	{
 		header->setMajorVersion((uint8_t) m_state->m_version);
-		header->setKind(m_state->m_isSpreadsheet ? WPS_SPREADSHEET : WPS_DATABASE);
+		header->setCreator(creator);
+		header->setKind(kind);
+		header->setNeedEncoding(needEncoding);
 	}
 	return true;
 }
@@ -380,7 +478,20 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 bool WKS4Parser::readZones()
 {
 	RVNGInputStreamPtr input = getInput();
-	input->seek(6, librevenge::RVNG_SEEK_SET);
+	input->seek(0, librevenge::RVNG_SEEK_SET);
+	if (version()>=1000)
+	{
+		// error ok, we do no known how to parsed this structure
+		while (!input->isEnd())
+		{
+			if (!readZoneQuattro())
+				break;
+		}
+
+		ascii().addPos(input->tell());
+		ascii().addNote("Entries(UnknownZone):");
+		return false;
+	}
 
 	while (readZone()) ;
 
@@ -391,21 +502,23 @@ bool WKS4Parser::readZones()
 	if (!checkFilePosition(pos+4))
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readZones: cell header is too short\n"));
-		return false;
+		return m_spreadsheetParser->getNumSpreadsheets()>0;
 	}
 	int type = (int) libwps::readU16(input); // 1
 	int length = (int) libwps::readU16(input);
 	if (length)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readZones: parse breaks before ending\n"));
-		return false;
+		ascii().addPos(pos);
+		ascii().addNote("Entries(BAD):###");
+		return m_spreadsheetParser->getNumSpreadsheets()>0;
 	}
 
 	ascii().addPos(pos);
 	if (type != 1)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readZones: odd end cell type: %d\n", type));
-		ascii().addNote("__End###");
+		ascii().addNote("Entries(BAD):###");
 	}
 	else
 		ascii().addNote("__End");
@@ -439,17 +552,42 @@ bool WKS4Parser::readZone()
 	case 0:
 		switch (id)
 		{
-		case 0x1:
+		/* also
+		   32: symphony windows settings(144)
+		   37: password checksum(4)
+		   3c: query(127)
+		   3d: query name(16)
+		   3e: symphony print record (679)
+		   3f: printer name(16)
+		   40: symphony graph record (499)
+		   42: zoom(9)
+		   43: number of split windows(2)
+		   44: number of screen row(2)
+		   45: number of screen column(2)
+		   46: name ruler range(25)
+		   47: name sheet range(25)
+		   48: autoload comm(65)
+		   49: autoexecutute macro adress(8)
+		   4a: query parse information
+		 */
+		case 0:
+			if (sz!=2) break;
+			f.str("");
+			f << "version=" << std::hex << libwps::readU16(input) << std::dec << ",";
+			isParsed=needWriteInAscii=true;
+			break;
+		case 0x1: // EOF
 			ok = false;
 			break;
 		// boolean
-		case 0x2: // 0|FF and one time 1
-		case 0x3: // always 0
-		case 0x4:
-		case 0x5: // always true
-		case 0x29: // 22|27|5e (spreadsheet)
-		case 0x30: // 0|ff (spreadsheet)
-		case 0x31: // 1|2
+		case 0x2: // Calculation mode 0 or FF
+		case 0x3: // Calculation order
+		case 0x4: // Split window type
+		case 0x5: // Split window syn
+		case 0x29: // label format 22|27|5e (spreadsheet)
+		case 0x30: // formatted/unformatted print 0|ff (spreadsheet)
+		case 0x31: // cursor/location 1|2
+		case 0x38: // lock
 			f.str("");
 			f << "Entries(Byte" << std::hex << id << std::dec << "Z):";
 			if (sz!=1) break;
@@ -468,20 +606,20 @@ bool WKS4Parser::readZone()
 			}
 			isParsed=needWriteInAscii=true;
 			break;
-		case 0x6:
+		case 0x6: // active worksheet range
 			ok = m_spreadsheetParser->readSheetSize();
 			isParsed = true;
 			break;
-		case 0x7:
-		case 0x9:
-			ok = readUnknown0();
+		case 0x7: // window 1 record
+		case 0x9: // window 2 record
+			ok = readWindowRecord();
 			isParsed=true;
 			break;
-		case 0x8:
+		case 0x8: // col width
 			ok = m_spreadsheetParser->readColumnSize();
 			isParsed = true;
 			break;
-		case 0xa: // find in a spreadsheet v1 ( duplicated )
+		case 0xa: // col width (window 2)
 			if (sz!=3) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			// varies in this file from 0 to 5
@@ -490,56 +628,56 @@ bool WKS4Parser::readZone()
 			f << "dim?=" << libwps::read8(input) << ",";
 			isParsed=needWriteInAscii=true;
 			break;
-		case 0xb:
+		case 0xb: // named range
 			ok = readFieldName();
 			isParsed=true;
 			break;
-		case 0xc:
-		case 0xd:
-		case 0xe:
-		case 0xf:
-		case 0x10:
+		case 0xc: // blank cell
+		case 0xd: // integer cell
+		case 0xe: // floating cell
+		case 0xf: // label cell
+		case 0x10: // formula cell
 			ok = m_spreadsheetParser->readCell();
 			isParsed = true;
 			break;
-		case 0x33:
+		case 0x33:  // value of string formula
 			ok = m_spreadsheetParser->readCellFormulaResult();
 			isParsed = true;
 			break;
 		// some spreadsheet zone ( mainly flags )
-		case 0x18:
-		case 0x19:
-		case 0x20:
-		case 0x27:
-		case 0x2a:
+		case 0x18: // data table range
+		case 0x19: // query range
+		case 0x20: // distribution range
+		case 0x27: // print setup
+		case 0x2a: // print borders
 			ok = readUnknown1();
 			isParsed=true;
 			break;
-		case 0x1a:
-		case 0x1b:
-		case 0x1c:
-		case 0x1d:
-		case 0x23:
+		case 0x1a: // print range
+		case 0x1b: // sort range
+		case 0x1c: // fill range
+		case 0x1d: // primary sort key range
+		case 0x23: // secondary sort key range
 		{
 			int expectedSz=8;
 			f.str("");
 			switch (id)
 			{
 			case 0x1a: // only in spreadsheet?
-				f << "Entries(Select0):";
+				f << "Entries(PrintRange):";
 				break;
 			case 0x1b: // a dimension or also some big selection? 31999=infinity?, related to report?
-				f << "Entries(Dim0):";
+				f << "Entries(SortRange):";
 				break;
 			case 0x1c: // a dimension or also some big selection? only in spreadsheet
-				f << "Entries(Dim1):";
+				f << "Entries(FillRange):";
 				break;
 			case 0x1d:
-				f << "Entries(Select1):";
+				f << "Entries(PrimSort):";
 				expectedSz=9;
 				break;
 			case 0x23:
-				f << "Entries(Sort)[function]:";
+				f << "Entries(SecSort):";
 				expectedSz=9;
 				break;
 			default:
@@ -572,9 +710,9 @@ bool WKS4Parser::readZone()
 			isParsed=needWriteInAscii=true;
 			break;
 		}
-		case 0x24:
+		case 0x24: // protection (global)
 			f.str("");
-			f << "Entries(Protection)[database]:";
+			f << "Entries(Protection):global,";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			val=(int) libwps::readU8(input);
@@ -587,12 +725,12 @@ bool WKS4Parser::readZone()
 			else if (val) f << "#protected=" << val << ",";
 			isParsed=needWriteInAscii=true;
 			break;
-		case 0x25:
-		case 0x26:
+		case 0x25: // footer
+		case 0x26: // header
 			readHeaderFooter(id==0x26);
 			isParsed = true;
 			break;
-		case 0x28: // only in spreadsheet ?
+		case 0x28: // print margin
 			if (sz!=10) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<5; ++i)   // f1=4c|96|ac|f0
@@ -603,22 +741,23 @@ bool WKS4Parser::readZone()
 			}
 			isParsed=needWriteInAscii=true;
 			break;
-		case 0x2d:
-		case 0x2e:
+		case 0x2d: // graph setting
+		case 0x2e: // named graph setting
 			readChartDef();
 			isParsed = true;
 			break;
-		case 0x2f: // first microsoft works file ?
+		case 0x2f: // iteration count: only in dos file Wk1, Wks(dos), Wq[12] ?
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			WPS_DEBUG_MSG(("WKS4Parser::readZone: arggh a WKS1 file?\n"));
 			f.str("");
-			f << "Entries(PreVersion):vers=" << (int) libwps::readU8(input);
+			val=(int) libwps::readU8(input);
+			f << "Entries(ItCount):dos";
+			if (val!=1) f << "=" << val << ",";
 			if (m_state->m_version==2)
 				m_state->m_version=1;
 			isParsed = needWriteInAscii = true;
 			break;
-		case 0x36:   // find one time in a spreadsheet
+		case 0x36: // find one time in a spreadsheet (not standard)
 		{
 			if (sz!=0x1e) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
@@ -631,10 +770,14 @@ bool WKS4Parser::readZone()
 			isParsed = needWriteInAscii = true;
 			break;
 		}
-		case 0x41:
+		case 0x41: // graph record name
 			readChartName();
 			isParsed = true;
 			break;
+		case 0x64: // hidden column
+			isParsed = m_spreadsheetParser->readHiddenColumns();
+			break;
+
 		default:
 			break;
 		}
@@ -644,19 +787,15 @@ bool WKS4Parser::readZone()
 		{
 		// always empty ?
 		case 0x25:
+			f.str("");
+			f << "Entries(LICS):";
 			if (sz)
 			{
 				f << "###";
-				WPS_DEBUG_MSG(("WKS4Parser::readZone: find a not empty %d zone\n", id));
-				break;
-			}
-			f.str("");
-			switch (id)
-			{
-			default:
-				f << "Entries(EndA" << std::hex << id << std::dec << "):";
+				WPS_DEBUG_MSG(("WKS4Parser::readZone: find a not empty LICS encoding zone\n"));
 				break;
 			}
+			m_state->m_hasLICSCharacters = true;
 			isParsed = needWriteInAscii = true;
 			break;
 		// boolean
@@ -736,7 +875,7 @@ bool WKS4Parser::readZone()
 			break;
 		}
 		case 0x2:
-			ok = m_spreadsheetParser->readDOSCellProperty();
+			ok = m_spreadsheetParser->readMsWorksDOSCellProperty();
 			isParsed = true;
 			break;
 		case 0x5:
@@ -747,7 +886,7 @@ bool WKS4Parser::readZone()
 			isParsed = needWriteInAscii = true;
 			break;
 		case 0x6:
-			ok = m_spreadsheetParser->readDOSFieldProperty();
+			ok = m_spreadsheetParser->readMsWorksDOSFieldProperty();
 			isParsed = true;
 			break;
 		case 0x8: // only in database?, checkme: the structure may be different in dosfile
@@ -787,7 +926,7 @@ bool WKS4Parser::readZone()
 			isParsed = true;
 			break;
 		case 0x13:
-			ok = m_spreadsheetParser->readPageBreak();
+			ok = m_spreadsheetParser->readMsWorksPageBreak();
 			isParsed = true;
 			break;
 		case 0x14:
@@ -841,28 +980,23 @@ bool WKS4Parser::readZone()
 		/* case 1b: 000000000000000000000000000000000000010000000000020000000000000000000000000000000000
 		   database v1 */
 		case 0x1c:
-			m_spreadsheetParser->readDOSCellExtraProperty();
+			m_spreadsheetParser->readMsWorksDOSCellExtraProperty();
 			isParsed = true;
 			break;
 		case 0x1f:
 		{
 			// frequent field, near the beginning of the file
-			// find with 050300000005|05000000, so maybe:
-			if (sz<4) break;
+			// find with 050300000005|058000000005|05000000, so maybe:
+			if (sz<4 || (sz%2)!=0) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input); // always 5?
-			if (val!=5) f << "unkn=" << val << ",";
-			int dSz=(int) libwps::readU16(input);
-			if (dSz+3 != sz && dSz+4!= sz)
+			val=(int) libwps::read8(input); // always 5?
+			if (val!=5) f << "f0=" << val << ",";
+			val=(int) libwps::readU8(input); // 0|80
+			if (val) f << "f1=" << std::hex << val << ",";
+			for (int i=1; i<sz/2; ++i)
 			{
-				WPS_DEBUG_MSG(("WKS4Parser::readZone: find unexpected data size for field A1f\n"));
-				f << "###";
-				break;
-			}
-			for (int i=0; i<dSz; ++i)
-			{
-				val=(int) libwps::read8(input);
-				if (val) f << "f" << i << "=" << val << ",";
+				val=(int) libwps::read16(input);
+				if (val) f << "f" << i+2 << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
 			break;
@@ -883,7 +1017,7 @@ bool WKS4Parser::readZone()
 			isParsed=needWriteInAscii=true;
 			break;
 		case 0x27:
-			ok = m_spreadsheetParser->readDOSPageBreak();
+			ok = m_spreadsheetParser->readMsWorksDOSPageBreak();
 			isParsed = true;
 			break;
 		case 0x33:
@@ -958,7 +1092,7 @@ bool WKS4Parser::readZone()
 			break;
 		}
 		case 0x5a:
-			ok = m_spreadsheetParser->readStyle();
+			ok = m_spreadsheetParser->readMsWorksStyle();
 			isParsed = true;
 			break;
 		case 0x5b:
@@ -1001,7 +1135,8 @@ bool WKS4Parser::readZone()
 				f << "##type=" << zType << ",";
 				break;
 			}
-			ascii().addDelimiter(input->tell(),'|');
+			if (input->tell()!=pos+4+sz)
+				ascii().addDelimiter(input->tell(),'|');
 			isParsed=needWriteInAscii=true;
 			break;
 		}
@@ -1024,7 +1159,7 @@ bool WKS4Parser::readZone()
 			break;
 		}
 		case 0x65:
-			ok = m_spreadsheetParser->readRowSize2();
+			ok = m_spreadsheetParser->readMsWorksRowSize();
 			isParsed = true;
 			break;
 		// case 66: ff|12c|13B|1d1, dim/flag? (database)
@@ -1034,7 +1169,7 @@ bool WKS4Parser::readZone()
 			isParsed = true;
 			break;
 		case 0x6b:
-			ok = m_spreadsheetParser->readColumnSize2();
+			ok = m_spreadsheetParser->readMsWorksColumnSize();
 			isParsed = true;
 			break;
 		case 0x6e: // field(series)
@@ -1086,6 +1221,33 @@ bool WKS4Parser::readZone()
 		return true;
 	}
 
+	if (sz && input->tell()!=pos && input->tell()!=pos+4+sz)
+		ascii().addDelimiter(input->tell(),'|');
+	input->seek(pos+4+sz, librevenge::RVNG_SEEK_SET);
+	ascii().addPos(pos);
+	ascii().addNote(f.str().c_str());
+	return true;
+}
+
+////////////////////////////////////////////////////////////
+//   other formats
+////////////////////////////////////////////////////////////
+bool WKS4Parser::readZoneQuattro()
+{
+	libwps::DebugStream f;
+	RVNGInputStreamPtr input = getInput();
+	long pos = input->tell();
+	int id = (int) libwps::readU8(input);
+	int type = (int) libwps::readU8(input);
+	long sz = (long) libwps::readU16(input);
+	if (type>5 || sz<0 || !checkFilePosition(pos+4+sz))
+	{
+		input->seek(pos, librevenge::RVNG_SEEK_SET);
+		return false;
+	}
+	f << "Entries(Quattro";
+	if (type) f << type << "A";
+	f << std::hex << id << std::dec << "E):";
 	if (sz) ascii().addDelimiter(pos+4,'|');
 	input->seek(pos+4+sz, librevenge::RVNG_SEEK_SET);
 	ascii().addPos(pos);
@@ -1118,7 +1280,7 @@ bool WKS4Parser::readFont()
 		return true;
 	}
 
-	WKS4ParserInternal::Font font;
+	WKS4ParserInternal::Font font(getDefaultFontType());
 	int flags = (int)libwps::readU8(input);
 	uint32_t attributes = 0;
 	if (flags & 1) attributes |= WPS_BOLD_BIT;
@@ -1138,17 +1300,17 @@ bool WKS4Parser::readFont()
 
 	int val=(int)libwps::readU8(input);
 	if (val) f << "f0=" << std::hex << val << std::dec << ",";
-	std::string name("");
+	librevenge::RVNGString name("");
 	while (int(input->tell()) < endPos-4)
 	{
 		char c = (char) libwps::readU8(input);
 		if (c == '\0') break;
-		name+= c;
+		name.append(c);
 	}
 
 	font.m_type=libwps_tools_win::Font::getFontType(name);
 	if (font.m_type==libwps_tools_win::Font::UNKNOWN)
-		font.m_type=version()<=2 ? libwps_tools_win::Font::DOS_850 : libwps_tools_win::Font::WIN3_WEUROPE;
+		font.m_type=getDefaultFontType();
 	font.m_name=name;
 
 	input->seek(endPos-4, librevenge::RVNG_SEEK_SET);
@@ -1157,6 +1319,10 @@ bool WKS4Parser::readFont()
 	int fSize = libwps::read16(input)/2;
 	if (fSize >= 1 && fSize <= 50)
 		font.m_size=double(fSize);
+	else
+		f << "###fSize=" << fSize << ",";
+	if (name.empty())
+		f << "###noName,";
 	font.m_extra=f.str();
 
 	f.str("");
@@ -1164,9 +1330,6 @@ bool WKS4Parser::readFont()
 	f << "font" << m_state->m_fontsList.size() << "[" << font << "]";
 	m_state->m_fontsList.push_back(font);
 
-	if (fSize == 0 && fSize > 50)
-		f << "###fSize=" << fSize;
-	if (name.length() <= 0) f << "###";
 	ascii().addPos(pos);
 	ascii().addNote(f.str().c_str());
 
@@ -1185,16 +1348,21 @@ void WKS4Parser::sendHeaderFooter(bool header)
 	}
 
 	m_listener->setFont(m_state->getDefaultFont());
-	libwps_tools_win::Font::Type fontType=version()<=2 ? libwps_tools_win::Font::DOS_850 : libwps_tools_win::Font::WIN3_WEUROPE;
+	libwps_tools_win::Font::Type fontType=m_state->getDefaultFontType();
 
 	std::string const &text=header ? m_state->m_headerString : m_state->m_footerString;
+	bool hasLICS=hasLICSCharacters();
 	for (size_t i=0; i < text.size(); ++i)
 	{
 		unsigned char c=(unsigned char) text[i];
 		if (c==0xd)
 			m_listener->insertEOL();
-		else if (c!=0xa)
+		else if (c==0xa)
+			continue;
+		else if (!hasLICS)
 			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType));
+		else
+			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c,fontType));
 	}
 
 }
@@ -1214,6 +1382,15 @@ bool WKS4Parser::readHeaderFooter(bool header)
 	long endPos = pos+4+sz;
 
 	f << "Entries(" << (header ? "HeaderText" : "FooterText") << "):";
+	if (sz==1)
+	{
+		// followed with 0
+		int val=(int) libwps::read8(input);
+		if (val) f << "##f0=" << val << ",";
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+		return true;
+	}
 	if (sz < 0xF2)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readHeaderFooter: the header/footer size seeem odds\n"));
@@ -1347,16 +1524,18 @@ bool WKS4Parser::readFieldName()
 		return false;
 	}
 	long sz = (long) libwps::readU16(input);
-	if (sz != 0x18)
+	f << "Entries(FldNames):";
+	if (sz != 0x18 && sz != 0x1e)
 	{
+		// find also 0x85 a zone with 4 fldnames ?
 		WPS_DEBUG_MSG(("WKS4Parser::readFieldName: size seems bad\n"));
+		f << "###";
 		ascii().addPos(pos);
-		ascii().addNote("Entries(ZoneB):###");
+		ascii().addNote(f.str().c_str());
 		return true;
 	}
-	f << "Entries(FldNames):";
 	std::string name("");
-	for (int i=0; i < 26; ++i)
+	for (int i=0; i < 16; ++i)
 	{
 		char c=(char) libwps::read8(input);
 		if (c=='\0') break;
@@ -1367,7 +1546,20 @@ bool WKS4Parser::readFieldName()
 	input->seek(pos+20, librevenge::RVNG_SEEK_SET);
 	// the position
 	int dim[4];
-	for (int i=0; i<4; ++i) dim[i]=(int) libwps::read16(input);
+	if (sz==0x18)
+	{
+		for (int i=0; i<4; ++i) dim[i]=(int) libwps::read16(input);
+	}
+	else
+	{
+		for (int i=0; i<7; ++i)
+		{
+			int val=(int) libwps::read16(input);
+			if (i<2) dim[i]=val;
+			else if (i>=3 && i<5) dim[i-1]=val;
+			else if (val) f << "f" << i << "=" << val << ",";
+		}
+	}
 	// in a spreadsheet, the cell or the cells corresponding to the field
 	// in a database, col,0,col,0xFFF
 	if (m_state->m_isSpreadsheet || dim[1] || dim[0]!= dim[2] || dim[3]!=0xFFF)
@@ -1765,7 +1957,7 @@ bool WKS4Parser::readChartList()
 
 
 /* the zone 0:7 and 0:9 */
-bool WKS4Parser::readUnknown0()
+bool WKS4Parser::readWindowRecord()
 {
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
@@ -1774,7 +1966,7 @@ bool WKS4Parser::readUnknown0()
 	long type = (long) libwps::read16(input);
 	if (type != 7 && type != 9)
 	{
-		WPS_DEBUG_MSG(("WKS4Parser::readUnknown0: unknown type\n"));
+		WPS_DEBUG_MSG(("WKS4Parser::readWindowRecord: unknown type\n"));
 		return false;
 	}
 	long sz = (long) libwps::readU16(input);
@@ -1782,13 +1974,13 @@ bool WKS4Parser::readUnknown0()
 	// normally size=0x1f but one time 0x1e
 	if (sz < 0x1e)
 	{
-		WPS_DEBUG_MSG(("WKS4Parser::readUnknown0: zone seems too short\n"));
+		WPS_DEBUG_MSG(("WKS4Parser::readWindowRecord: zone seems too short\n"));
 		ascii().addPos(pos);
-		ascii().addNote("Entries(Unknown0):###");
+		ascii().addNote("Entries(WindowRecord):###");
 		return true;
 	}
 
-	f << "Entries(Unknown0)[" << type << "]:";
+	f << "Entries(WindowRecord)[" << type << "]:";
 	// f0=0-a|1f|21, f1=1-3c|1da, f2=0-6|71|7f|f1, f3=4|9|a|c(size?),
 	// f4=0|4|6-11, f5=5-2c, f6=0|3|6|10|1f|20, f7=0-3c|1ca (related to f1?)
 	// f8=0|1, f9=0|2, f10=f11=0
@@ -1845,6 +2037,14 @@ bool WKS4Parser::readUnknown1()
 	f << "Entries(Flags" << std::hex << type << std::dec << ")]:";
 	if (sz != expectedSize+extraSize)
 	{
+		// find also 270001000[01]
+		if (type==0x27 && sz==1)
+		{
+			f << "f0=" << (int) libwps::read8(input) << ",";
+			ascii().addPos(pos);
+			ascii().addNote(f.str().c_str());
+			return true;
+		}
 		WPS_DEBUG_MSG(("WKS4Parser::readUnknown1: the zone size seems too bad\n"));
 		f << "###";
 		ascii().addPos(pos);
