@@ -127,7 +127,7 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 //! the state of WPS4
 struct State
 {
-	State(libwps_tools_win::Font::Type fontType) :
+	explicit State(libwps_tools_win::Font::Type fontType) :
 		m_isDosFile(false), m_eof(-1), m_fontType(fontType),
 		m_OEMFontType(libwps_tools_win::Font::UNKNOWN),
 		m_pageSpan(), m_noFirstPageHeader(false), m_noFirstPageFooter(false),
@@ -631,7 +631,7 @@ bool WPS4Parser::findZones()
 		if (type != libwps_tools_win::Font::UNKNOWN)
 		{
 			m_state->m_OEMFontType = type;
-			f << "codePage=" << libwps_tools_win::Font::getTypeName(type) << ",";
+			f << "codePage=" << libwps_tools_win::Font::getTypeName(type).cstr() << ",";
 		}
 		else
 			f << "#codePage=" << std::hex << ((oem>>4)&0x7ff) << std::dec << ",";
@@ -976,7 +976,7 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	// Find in one file str[0]="C:\Databases\Elem 02-03.wdb",str[120]="Query1"
 	while (pos++ < 0x132)   // try to find some strings
 	{
-		char c = libwps::read8(input);
+		char c = char(libwps::read8(input));
 		if (c == '\0')
 		{
 			if (debSPos>=0)
