@@ -87,32 +87,31 @@ enum ZoneType { Z_String=-1, Z_Header=0, Z_Footer=1, Z_Main=2, Z_Note, Z_Bookmar
 struct FontName
 {
 	//! constructor with file's version to define the default encoding */
-	FontName(int version=4) : m_name(""), m_type(libwps_tools_win::Font::WIN3_WEUROPE)
+	explicit FontName(libwps_tools_win::Font::Type type) : m_name(""), m_type(type)
 	{
-		if (version <= 2) m_type = libwps_tools_win::Font::DOS_850;
 	}
 	//! operator<<
 	friend std::ostream &operator<<(std::ostream &o, FontName const &ft);
 	/** returns the default dos name corresponding to \a id th font */
-	static std::string getDosName(int id);
+	static librevenge::RVNGString getDosName(int id);
 
 	//! font name
-	std::string m_name;
+	librevenge::RVNGString m_name;
 	//! font encoding type
 	libwps_tools_win::Font::Type m_type;
 };
 //! operator<< for a font name
 std::ostream &operator<<(std::ostream &o, FontName const &ft)
 {
-	if (!ft.m_name.empty()) o << "name='" << ft.m_name << "'";
+	if (!ft.m_name.empty()) o << "name='" << ft.m_name.cstr() << "'";
 	else o << "name='Unknown'";
 	if (ft.m_type!=libwps_tools_win::Font::WIN3_WEUROPE &&
 	        ft.m_type!=libwps_tools_win::Font::DOS_850)
-		o << ",type=" << libwps_tools_win::Font::getTypeName(ft.m_type) << ",";
+		o << ",type=" << libwps_tools_win::Font::getTypeName(ft.m_type).cstr() << ",";
 	return o;
 }
 
-std::string FontName::getDosName(int id)
+librevenge::RVNGString FontName::getDosName(int id)
 {
 	switch (id)
 	{
@@ -143,15 +142,14 @@ std::string FontName::getDosName(int id)
 struct Font : public WPSFont
 {
 	//! constructor with file's version to define the default encoding */
-	Font(int version=4) : WPSFont(), m_type(libwps_tools_win::Font::WIN3_WEUROPE),
-		m_backColor(0xFFFFFF), m_special(false), m_dlinkId(-1)
+	explicit Font(libwps_tools_win::Font::Type type) : WPSFont(), m_type(type),
+		m_backColor(WPSColor::white()), m_special(false), m_dlinkId(-1)
 	{
-		if (version <= 2) m_type = libwps_tools_win::Font::DOS_850;
 	}
 	//! returns a default font (Courier12) with file's version to define the default encoding */
-	static Font getDefault(int version)
+	static Font getDefault(libwps_tools_win::Font::Type type, int version)
 	{
-		Font res(version);
+		Font res(type);
 		if (version <= 2)
 			res.m_name="Courier";
 		else
@@ -166,7 +164,7 @@ struct Font : public WPSFont
 	//! the font encoding type
 	libwps_tools_win::Font::Type m_type;
 	//! background  color index
-	uint32_t m_backColor;
+	WPSColor m_backColor;
 	//! a flag to know if we have a special field (a note), ...
 	bool m_special;
 	//! a id to retrieve a file name ( dos )
@@ -186,7 +184,7 @@ std::ostream &operator<<(std::ostream &o, Font const &ft)
 			o << "spec,";
 	}
 
-	if (ft.m_backColor != 0xFFFFFF)
+	if (!ft.m_backColor.isWhite())
 		o << "bgCol=" << ft.m_backColor << ",";
 	return o;
 }
@@ -525,11 +523,9 @@ WPSEntry WPS4Text::getMainTextEntry() const
 	return m_state->m_main;
 }
 
-libwps_tools_win::Font::Type WPS4Text::getDefaultFontType() const
+WPS4TextInternal::Font WPS4Text::getDefaultFont() const
 {
-	if (version()<=2)
-		return libwps_tools_win::Font::DOS_850;
-	return libwps_tools_win::Font::WIN3_WEUROPE;
+	return WPS4TextInternal::Font::getDefault(mainParser().getDefaultFontType(), version());
 }
 
 ////////////////////////////////////////////////////////////
@@ -545,7 +541,7 @@ void WPS4Text::flushExtra()
 	size_t numExtra = m_state->m_otherZones.size();
 	if (numExtra == 0) return;
 
-	m_listener->setFont(WPS4TextInternal::Font::getDefault(version()));
+	m_listener->setFont(getDefaultFont());
 	m_listener->setParagraph(WPS4TextInternal::Paragraph());
 	m_listener->insertEOL();
 #ifdef DEBUG
@@ -613,14 +609,10 @@ bool WPS4Text::readText(WPSEntry const &zone)
 		else if (fod.m_type == DataFOD::ATTR_PARAG) prevPId = id;
 	}
 
-	WPS4TextInternal::Font actFont;
+	WPS4TextInternal::Font defaultFont(getDefaultFont());
+	WPS4TextInternal::Font actFont(defaultFont);
 	if (prevFId != -1)
 		actFont = m_state->m_fontList[size_t(prevFId)];
-	else
-	{
-		actFont = WPS4TextInternal::Font::getDefault(version());
-		actFont.m_type=getDefaultFontType();
-	}
 	m_listener->setFont(actFont);
 
 	if (prevPId != -1)
@@ -671,10 +663,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 				if (fId >= 0)
 					actFont = m_state->m_fontList[size_t(fId)];
 				else
-				{
-					actFont = WPS4TextInternal::Font::getDefault(version());
-					actFont.m_type=getDefaultFontType();
-				}
+					actFont = defaultFont;
 				m_listener->setFont(actFont);
 #if DEBUG_FP
 				f << "[";
@@ -747,7 +736,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			case 0x08: // spreadsheet range
 			case 0x0e: // picture
 			{
-				if (!actFont.m_special || actFont.m_dlinkId >= int(m_state->m_dosLinkList.size()))
+				if (!actFont.m_special || m_state->m_dosLinkList.empty() || actFont.m_dlinkId >= int(m_state->m_dosLinkList.size()))
 				{
 					WPS_DEBUG_MSG(("WPS4Text::readText: send DLINK can not find id\n"));
 					break;
@@ -765,22 +754,22 @@ bool WPS4Text::readText(WPSEntry const &zone)
 				break;
 			}
 			case 0x02:
-				m_listener->insertField(WPSContentListener::PageNumber);
+				m_listener->insertField(WPSField(WPSField::PageNumber));
 				break;
 			case 0x03:
-				m_listener->insertField(WPSContentListener::Date);
+				m_listener->insertField(WPSField(WPSField::Date));
 				break;
 			case 0x04:
-				m_listener->insertField(WPSContentListener::Time);
+				m_listener->insertField(WPSField(WPSField::Time));
 				break;
 			case 0x05:
-				m_listener->insertField(WPSContentListener::Title);
+				m_listener->insertField(WPSField(WPSField::Title));
 				break;
 			case 0x07:
 			{
 				if (m_state->m_objectMap.find(actPos) == m_state->m_objectMap.end())
 				{
-					WPS_DEBUG_MSG(("WPS4Text::readText: can not find object for position : %lX\n", actPos));
+					WPS_DEBUG_MSG(("WPS4Text::readText: can not find object for position : %lX\n", (unsigned long) actPos));
 				}
 				else
 				{
@@ -795,17 +784,21 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			{
 				if (m_state->m_dateTimeMap.find(actPos) == m_state->m_dateTimeMap.end())
 				{
-					WPS_DEBUG_MSG(("WPS4Text::readText: can not find date/time for position : %lX\n", actPos));
+					WPS_DEBUG_MSG(("WPS4Text::readText: can not find date/time for position : %lX\n", (unsigned long) actPos));
 				}
 				else
 				{
 					WPS4TextInternal::DateTime const &form = m_state->m_dateTimeMap[actPos];
 					std::string format = form.format();
 					if (format.length())
-						m_listener->insertDateTimeField(format.c_str());
+					{
+						WPSField field(WPSField::Date);
+						field.m_DTFormat=format;
+						m_listener->insertField(field);
+					}
 					else
 					{
-						WPS_DEBUG_MSG(("WPS4Text::readText: unknown date/time format for position : %lX\n", actPos));
+						WPS_DEBUG_MSG(("WPS4Text::readText: unknown date/time format for position : %lX\n", (unsigned long) actPos));
 					}
 				}
 				break;
@@ -855,19 +848,19 @@ bool WPS4Text::readText(WPSEntry const &zone)
 					{
 					case 'p':
 					case 'P':
-						m_listener->insertField(WPSContentListener::PageNumber);
+						m_listener->insertField(WPSField(WPSField::PageNumber));
 						break;
 					case 'd':
 					case 'D':
-						m_listener->insertField(WPSContentListener::Date);
+						m_listener->insertField(WPSField(WPSField::Date));
 						break;
 					case 't':
 					case 'T':
-						m_listener->insertField(WPSContentListener::Time);
+						m_listener->insertField(WPSField(WPSField::Time));
 						break;
 					case 'f':
 					case 'F':
-						m_listener->insertField(WPSContentListener::Title);
+						m_listener->insertField(WPSField(WPSField::Title));
 						break;
 					// case '&': check me does '&&'->'&' ?
 					default:
@@ -1308,7 +1301,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 
 	long endPos = entry.end();
 	int nFonts = 0;
-	libwps_tools_win::Font::Type docType=getDefaultFontType();
+	libwps_tools_win::Font::Type docType=mainParser().getDefaultFontType();
 	while (m_input->tell() < endPos)
 	{
 		long actPos;
@@ -1321,7 +1314,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 		if (m_state->m_fontNames.find(font_number) != m_state->m_fontNames.end())
 		{
 			WPS_DEBUG_MSG(("WPS4Text::readFontNames: at position 0x%lx: font number %i duplicated\n",
-			               (m_input->tell())-2, font_number));
+			               (unsigned long)(m_input->tell()-2), font_number));
 			throw libwps::ParseException();
 		}
 
@@ -1330,7 +1323,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 		uint8_t unknown_byte = libwps::readU8(m_input);
 		f << "unk=" << (int)unknown_byte << ", ";
 
-		std::string s;
+		librevenge::RVNGString s;
 		uint8_t nChar = libwps::readU8(m_input);
 		for (uint8_t i = nChar; i>0; i--)
 		{
@@ -1342,7 +1335,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 			}
 			unsigned char val = libwps::readU8(m_input);
 			// sanity check (because sometimes contains char > 0x80 .. )
-			if (val >= ' ' && val <= 'z') s.append(1,char(val));
+			if (val >= ' ' && val <= 'z') s.append(char(val));
 			else
 			{
 				static bool first = true;
@@ -1357,12 +1350,11 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 		libwps_tools_win::Font::Type fType=libwps_tools_win::Font::getFontType(s);
 		if (fType==libwps_tools_win::Font::UNKNOWN)
 			fType=docType;
-		WPS4TextInternal::FontName font;
+		WPS4TextInternal::FontName font(fType);
 		font.m_name = s;
-		font.m_type = fType;
 		f << font;
 
-		m_state->m_fontNames[font_number] = font;
+		m_state->m_fontNames.insert(std::pair<int,WPS4TextInternal::FontName>(font_number,font));
 
 		ascii().addPos(actPos);
 		ascii().addNote(f.str().c_str());
@@ -1377,7 +1369,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 ////////////////////////////////////////////////////////////
 bool WPS4Text::readFont(long endPos, int &id, std::string &mess)
 {
-	WPS4TextInternal::Font font(version());
+	WPS4TextInternal::Font font(getDefaultFont());
 	font.m_size = 12;
 
 	libwps::DebugStream f;
@@ -1416,13 +1408,14 @@ bool WPS4Text::readFont(long endPos, int &id, std::string &mess)
 
 		if (m_state->m_fontNames.find(font_n) != m_state->m_fontNames.end())
 		{
-			font.m_name=m_state->m_fontNames[font_n].m_name;
-			font.m_type=m_state->m_fontNames[font_n].m_type;
+			WPS4TextInternal::FontName const &fontName=m_state->m_fontNames.find(font_n)->second;
+			font.m_name=fontName.m_name;
+			font.m_type=fontName.m_type;
 		}
 		else if (version() <= 2)
 		{
 			font.m_name=WPS4TextInternal::FontName::getDosName(font_n);
-			font.m_type=getDefaultFontType();
+			font.m_type=mainParser().getDefaultFontType();
 		}
 		else
 		{
@@ -1482,7 +1475,7 @@ bool WPS4Text::readFont(long endPos, int &id, std::string &mess)
 		}
 		if (setColor)
 		{
-			uint32_t color;
+			WPSColor color;
 			if (mainParser().getColor(bkColor, color))
 				font.m_backColor = color;
 			if (mainParser().getColor(ftColor, color))
@@ -1564,6 +1557,7 @@ bool WPS4Text::readDosLink(WPSEntry const &entry)
 			val = libwps::readU16(m_input); // always 4
 			if (val != 4) f << "g1=" << val << ",";
 		}
+		// fall-through intended
 		case 0x40: // spreadsheet range
 		case 0x01: // char ?
 		{
@@ -1693,7 +1687,8 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				break;
 			}
 			arg = libwps::readU8(m_input);
-			pp.m_borderStyle.m_style = WPSBorder::Single;
+			pp.m_borderStyle.m_style = WPSBorder::Simple;
+			pp.m_borderStyle.m_type = WPSBorder::Single;
 			pp.m_borderStyle.m_width = 1;
 			int style = (arg&0xf);
 			switch (style)
@@ -1704,7 +1699,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				pp.m_borderStyle.m_width = 2;
 				break;
 			case 2:
-				pp.m_borderStyle.m_style = WPSBorder::Double;
+				pp.m_borderStyle.m_type = WPSBorder::Double;
 				break;
 			case 3:
 				pp.m_borderStyle.m_style = WPSBorder::Dot;
@@ -1723,7 +1718,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 			case 9:
 			case 10:
 				pp.m_borderStyle.m_width = style-7;
-				pp.m_borderStyle.m_style = WPSBorder::Double;
+				pp.m_borderStyle.m_type = WPSBorder::Double;
 				break;
 			default:
 				f << "#borderStyle=" << style << ",";
@@ -1733,7 +1728,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 			int high = (arg>>4);
 			if (version() < 3)
 			{
-				uint32_t color;
+				WPSColor color;
 				if (high && mainParser().getColor(high, color))
 					pp.m_borderStyle.m_color = color;
 				else if (high)
@@ -1802,7 +1797,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				break;
 			}
 			int colorId = libwps::readU8(m_input);
-			uint32_t color;
+			WPSColor color;
 			if (mainParser().getColor(colorId, color))
 				pp.m_borderStyle.m_color = color;
 			else
@@ -1829,9 +1824,9 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				0x2713, 0x261e, 0x2704, 0x2611, 0x2612, 0x270e /* 18-24 */
 			};
 			if (arg <= 24 && bulletList[arg-1])
-				WPSContentListener::appendUnicode(bulletList[arg-1], pp.m_listLevel.m_bullet);
+				libwps::appendUnicode(bulletList[arg-1], pp.m_listLevel.m_bullet);
 			else
-				WPSContentListener::appendUnicode(0x2022, pp.m_listLevel.m_bullet);
+				libwps::appendUnicode(0x2022, pp.m_listLevel.m_bullet);
 			break;
 		}
 		case 0x1b:
@@ -1853,13 +1848,13 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 			case 0x1b:
 			{
 				if (arg==0) break;
-				uint32_t color;
+				WPSColor color;
 				if (mainParser().getColor(arg>>4, color))
-					f << "backPatternBackColor=" << std::hex << color << std::dec << ";";
+					f << "backPatternBackColor=" << color << ";";
 				else
 					f << "#backPatternBackColor=" << (arg>>4) << ",";
 				if (mainParser().getColor(arg&0xf, color))
-					f << "backPatternFrontColor=" << std::hex << color << std::dec << ";";
+					f << "backPatternFrontColor=" << color << ";";
 				else
 					f << "#backPatternFrontColor=" << (arg&0xf) << ",";
 				break;
@@ -1968,6 +1963,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				break;
 			case 0x13: // seems another way to define the left margin
 				f << "#left,";
+			// fall-through intended
 			case 0x12:
 				pp.m_margins[1] = arg/1440.;
 				break;
@@ -1976,12 +1972,13 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 				break;
 			case 0x15:
 			{
-				pp.m_spacings[0] = arg ? arg/240. : 1.0;
-				if (pp.m_spacings[0] < 1.0 || pp.m_spacings[0] > 2.0)
+				double lines = arg ? arg/240. : 1.0;
+				if (lines < 1.0 || lines > 2.0)
 				{
-					f << "##interLineSpacing=" << pp.m_spacings[0] << ",";
-					pp.m_spacings[0] = (pp.m_spacings[0] < 1.0) ? 1.0 : 2.0;
+					f << "##interLineSpacing=" << lines << ",";
+					lines = (lines < 1.0) ? 1.0 : 2.0;
 				}
+				pp.setInterline(lines, librevenge::RVNG_PERCENT);
 				break;
 			}
 			case 0x16:
@@ -2088,7 +2085,7 @@ bool WPS4Text::readFootNotes(WPSEntry const &ftnD, WPSEntry const &ftnP)
 			{
 				WPS_DEBUG_MSG
 				(("WPS4Text: error: can not create footnotes zone, found %lx and %lx\n",
-				  m_state->m_footnoteList[j].end(),m_state->m_footnoteList[j-1].begin()));
+				  (unsigned long) m_state->m_footnoteList[j].end(),(unsigned long) m_state->m_footnoteList[j-1].begin()));
 
 				m_state->m_footnoteList.resize(0);
 				return false;
@@ -2144,11 +2141,11 @@ bool WPS4Text::footNotesDataParser(long /*bot*/, long /*eot*/, int id,
 	{
 		int numC = type/2;
 		librevenge::RVNGString label("");
-		libwps_tools_win::Font::Type actType = getDefaultFontType();
+		libwps_tools_win::Font::Type actType = mainParser().getDefaultFontType();
 		for (int i=0; i < numC; ++i)
 		{
 			unsigned char c = libwps::readU8(m_input);
-			WPSContentListener::appendUnicode(uint32_t(libwps_tools_win::Font::unicode(c, actType)),label);
+			libwps::appendUnicode(uint32_t(libwps_tools_win::Font::unicode(c, actType)),label);
 			if (c < 0x20)
 				f << "#(" << std::hex << int(c) << std::dec << ")";
 		}
