@@ -1,33 +1,30 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* libwps
+ * Version: MPL 2.0 / LGPLv2.1+
+ *
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ *
+ * Major Contributor(s):
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006-2007 Fridrich Strba (fridrich.strba@bluewin.ch)
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * For minor contributions see the git repository.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+ * Alternatively, the contents of this file may be used under the terms
+ * of the GNU Lesser General Public License Version 2.1 or later
+ * (LGPLv2.1+), in which case the provisions of the LGPLv2.1+ are
+ * applicable instead of those above.
  *
  * For further information visit http://libwpd.sourceforge.net
  */
 
-/* "This product is not manufactured, approved, or supported by
- * Corel Corporation or Corel Corporation Limited."
- */
-#include <libwpd/WPXDocumentInterface.h>
-#include <libwpd/WPXProperty.h>
+#include <librevenge/librevenge.h>
 
 #include "libwps_internal.h"
+#include "WKSContentListener.h"
 #include "WPSContentListener.h"
 #include "WPSSubDocument.h"
 
@@ -39,13 +36,8 @@ namespace WPSPageSpanInternal
 class HeaderFooter
 {
 public:
-	HeaderFooter(const WPSPageSpan::HeaderFooterType headerFooterType, const WPSPageSpan::HeaderFooterOccurence occurence, WPSSubDocumentPtr &subDoc)  :
-		m_type(headerFooterType), m_occurence(occurence), m_subDocument(subDoc)
-	{
-	}
-
-	HeaderFooter(const HeaderFooter &headerFooter) :
-		m_type(headerFooter.getType()), m_occurence(headerFooter.getOccurence()), m_subDocument(headerFooter.m_subDocument)
+	HeaderFooter(WPSPageSpan::HeaderFooterType const &headerFooterType, WPSPageSpan::HeaderFooterOccurrence const &occurrence, WPSSubDocumentPtr &subDoc)  :
+		m_type(headerFooterType), m_occurrence(occurrence), m_subDocument(subDoc)
 	{
 	}
 
@@ -57,9 +49,9 @@ public:
 	{
 		return m_type;
 	}
-	WPSPageSpan::HeaderFooterOccurence getOccurence() const
+	WPSPageSpan::HeaderFooterOccurrence getOccurrence() const
 	{
-		return m_occurence;
+		return m_occurrence;
 	}
 	WPSSubDocumentPtr &getSubDocument()
 	{
@@ -72,7 +64,7 @@ public:
 	}
 private:
 	WPSPageSpan::HeaderFooterType m_type;
-	WPSPageSpan::HeaderFooterOccurence m_occurence;
+	WPSPageSpan::HeaderFooterOccurrence m_occurrence;
 	WPSSubDocumentPtr m_subDocument;
 };
 
@@ -81,7 +73,7 @@ bool HeaderFooter::operator==(shared_ptr<HeaderFooter> const &hF) const
 	if (!hF) return false;
 	if (m_type != hF.get()->m_type)
 		return false;
-	if (m_occurence != hF.get()->m_occurence)
+	if (m_occurrence != hF.get()->m_occurrence)
 		return false;
 	if (!m_subDocument)
 		return !hF.get()->m_subDocument;
@@ -110,36 +102,19 @@ WPSPageSpan::WPSPageSpan() :
 {
 }
 
-WPSPageSpan::WPSPageSpan(const WPSPageSpan &page) :
-	m_formLength(page.getFormLength()),
-	m_formWidth(page.getFormWidth()),
-	m_formOrientation(page.getFormOrientation()),
-	m_marginLeft(page.getMarginLeft()),
-	m_marginRight(page.getMarginRight()),
-	m_marginTop(page.getMarginTop()),
-	m_marginBottom(page.getMarginBottom()),
-	m_pageNumberPosition(page.getPageNumberPosition()),
-	m_pageNumber(page.getPageNumber()),
-	m_pageNumberingType(page.getPageNumberingType()),
-	m_pageNumberingFontName(page.getPageNumberingFontName()),
-	m_pageNumberingFontSize(page.getPageNumberingFontSize()),
-	m_headerFooterList(page.getHeaderFooterList()),
-	m_pageSpan(page.getPageSpan())
-{
-}
-
 WPSPageSpan::~WPSPageSpan()
 {
 }
 
-void WPSPageSpan::setHeaderFooter(const HeaderFooterType type, const HeaderFooterOccurence occurence,
+void WPSPageSpan::setHeaderFooter(const HeaderFooterType type, const HeaderFooterOccurrence occurrence,
                                   WPSSubDocumentPtr &subDocument)
 {
-	WPSPageSpanInternal::HeaderFooter headerFooter(type, occurence, subDocument);
-	switch (occurence)
+	WPSPageSpanInternal::HeaderFooter headerFooter(type, occurrence, subDocument);
+	switch (occurrence)
 	{
 	case NEVER:
 		_removeHeaderFooter(type, ALL);
+	case FIRST:
 	case ALL:
 		_removeHeaderFooter(type, ODD);
 		_removeHeaderFooter(type, EVEN);
@@ -150,9 +125,11 @@ void WPSPageSpan::setHeaderFooter(const HeaderFooterType type, const HeaderFoote
 	case EVEN:
 		_removeHeaderFooter(type, ALL);
 		break;
+	default:
+		break;
 	}
 
-	_setHeaderFooter(type, occurence, subDocument);
+	_setHeaderFooter(type, occurrence, subDocument);
 
 	bool containsHFLeft = _containsHeaderFooter(type, ODD);
 	bool containsHFRight = _containsHeaderFooter(type, EVEN);
@@ -172,8 +149,7 @@ void WPSPageSpan::setHeaderFooter(const HeaderFooterType type, const HeaderFoote
 	}
 }
 
-void WPSPageSpan::sendHeaderFooters(WPSContentListener *listener,
-                                    WPXDocumentInterface *documentInterface)
+void WPSPageSpan::sendHeaderFooters(WPSContentListener *listener, librevenge::RVNGTextInterface *documentInterface)
 {
 	if (!listener || !documentInterface)
 	{
@@ -182,22 +158,25 @@ void WPSPageSpan::sendHeaderFooters(WPSContentListener *listener,
 	}
 
 	bool pageNumberInserted = false;
-	for (int i = 0; i < int(m_headerFooterList.size()); i++)
+	for (size_t i = 0; i < m_headerFooterList.size(); i++)
 	{
 		WPSPageSpanInternal::HeaderFooterPtr &hf = m_headerFooterList[i];
 		if (!hf) continue;
 
-		WPXPropertyList propList;
-		switch (hf->getOccurence())
+		librevenge::RVNGPropertyList propList;
+		switch (hf->getOccurrence())
 		{
+		case WPSPageSpan::FIRST:
+			propList.insert("librevenge:occurrence", "first");
+			break;
 		case WPSPageSpan::ODD:
-			propList.insert("libwpd:occurence", "odd");
+			propList.insert("librevenge:occurrence", "odd");
 			break;
 		case WPSPageSpan::EVEN:
-			propList.insert("libwpd:occurence", "even");
+			propList.insert("librevenge:occurrence", "even");
 			break;
 		case WPSPageSpan::ALL:
-			propList.insert("libwpd:occurence", "all");
+			propList.insert("librevenge:occurrence", "all");
 			break;
 		case WPSPageSpan::NEVER:
 		default:
@@ -226,14 +205,14 @@ void WPSPageSpan::sendHeaderFooters(WPSContentListener *listener,
 		else
 			documentInterface->closeFooter();
 
-		WPS_DEBUG_MSG(("Header Footer Element: type: %i occurence: %i\n",
-		               hf->getType(), hf->getOccurence()));
+		WPS_DEBUG_MSG(("Header Footer Element: type: %i occurrence: %i\n",
+		               hf->getType(), hf->getOccurrence()));
 	}
 
 	if (!pageNumberInserted)
 	{
-		WPXPropertyList propList;
-		propList.insert("libwpd:occurence", "all");
+		librevenge::RVNGPropertyList propList;
+		propList.insert("librevenge:occurrence", "all");
 		if (m_pageNumberPosition >= TopLeft &&
 		        m_pageNumberPosition <= TopInsideLeftAndRight)
 		{
@@ -251,9 +230,57 @@ void WPSPageSpan::sendHeaderFooters(WPSContentListener *listener,
 	}
 }
 
-void WPSPageSpan::getPageProperty(WPXPropertyList &propList) const
+void WPSPageSpan::sendHeaderFooters(WKSContentListener *listener, librevenge::RVNGSpreadsheetInterface *documentInterface)
 {
-	propList.insert("libwpd:num-pages", getPageSpan());
+	if (!listener || !documentInterface)
+	{
+		WPS_DEBUG_MSG(("WPSPageSpan::sendHeaderFooters: no listener or document interface\n"));
+		return;
+	}
+
+	for (size_t i = 0; i < m_headerFooterList.size(); i++)
+	{
+		WPSPageSpanInternal::HeaderFooterPtr &hf = m_headerFooterList[i];
+		if (!hf) continue;
+
+		librevenge::RVNGPropertyList propList;
+		switch (hf->getOccurrence())
+		{
+		case WPSPageSpan::FIRST:
+			propList.insert("librevenge:occurrence", "first");
+			break;
+		case WPSPageSpan::ODD:
+			propList.insert("librevenge:occurrence", "odd");
+			break;
+		case WPSPageSpan::EVEN:
+			propList.insert("librevenge:occurrence", "even");
+			break;
+		case WPSPageSpan::ALL:
+			propList.insert("librevenge:occurrence", "all");
+			break;
+		case WPSPageSpan::NEVER:
+		default:
+			break;
+		}
+		bool isHeader = hf->getType() == WPSPageSpan::HEADER;
+		if (isHeader)
+			documentInterface->openHeader(propList);
+		else
+			documentInterface->openFooter(propList);
+		listener->handleSubDocument(hf->getSubDocument(), libwps::DOC_HEADER_FOOTER);
+		if (isHeader)
+			documentInterface->closeHeader();
+		else
+			documentInterface->closeFooter();
+
+		WPS_DEBUG_MSG(("Header Footer Element: type: %i occurrence: %i\n",
+		               hf->getType(), hf->getOccurrence()));
+	}
+}
+
+void WPSPageSpan::getPageProperty(librevenge::RVNGPropertyList &propList) const
+{
+	propList.insert("librevenge:num-pages", getPageSpan());
 
 	propList.insert("fo:page-height", getFormLength());
 	propList.insert("fo:page-width", getFormWidth());
@@ -272,11 +299,14 @@ bool WPSPageSpan::operator==(shared_ptr<WPSPageSpan> const &page2) const
 {
 	if (!page2) return false;
 	if (page2.get() == this) return true;
-	if (m_formLength != page2->m_formLength || m_formWidth != page2->m_formLength ||
+	if (m_formLength < page2->m_formLength || m_formLength > page2->m_formLength ||
+	        m_formWidth < page2->m_formWidth || m_formWidth > page2->m_formWidth ||
 	        m_formOrientation != page2->m_formOrientation)
 		return false;
-	if (getMarginLeft() != page2->getMarginLeft() || getMarginRight() != page2->getMarginRight() ||
-	        getMarginTop() != page2->getMarginTop()|| getMarginBottom() != page2->getMarginBottom())
+	if (getMarginLeft()<page2->getMarginLeft() || getMarginLeft()>page2->getMarginLeft() ||
+	        getMarginRight()<page2->getMarginRight() || getMarginRight()>page2->getMarginRight() ||
+	        getMarginTop()<page2->getMarginTop() || getMarginTop()>page2->getMarginTop() ||
+	        getMarginBottom()<page2->getMarginBottom() || getMarginBottom()>page2->getMarginBottom())
 		return false;
 
 	if (getPageNumberPosition() != page2->getPageNumberPosition())
@@ -289,23 +319,24 @@ bool WPSPageSpan::operator==(shared_ptr<WPSPageSpan> const &page2) const
 		return false;
 
 	if (getPageNumberingFontName() != page2->getPageNumberingFontName() ||
-	        getPageNumberingFontSize() != page2->getPageNumberingFontSize())
+	        getPageNumberingFontSize() < page2->getPageNumberingFontSize() ||
+	        getPageNumberingFontSize() > page2->getPageNumberingFontSize())
 		return false;
 
-	int numHF = m_headerFooterList.size();
-	int numHF2 = page2->m_headerFooterList.size();
-	for (int i = numHF; i < numHF2; i++)
+	size_t numHF = m_headerFooterList.size();
+	size_t numHF2 = page2->m_headerFooterList.size();
+	for (size_t i = numHF; i < numHF2; i++)
 	{
 		if (page2->m_headerFooterList[i])
 			return false;
 	}
-	for (int i = numHF2; i < numHF; i++)
+	for (size_t i = numHF2; i < numHF; i++)
 	{
 		if (m_headerFooterList[i])
 			return false;
 	}
 	if (numHF2 < numHF) numHF = numHF2;
-	for (int i = 0; i < numHF; i++)
+	for (size_t i = 0; i < numHF; i++)
 	{
 		if (!m_headerFooterList[i])
 		{
@@ -323,9 +354,9 @@ bool WPSPageSpan::operator==(shared_ptr<WPSPageSpan> const &page2) const
 	return true;
 }
 
-void WPSPageSpan::_insertPageNumberParagraph(WPXDocumentInterface *documentInterface)
+void WPSPageSpan::_insertPageNumberParagraph(librevenge::RVNGTextInterface *documentInterface)
 {
-	WPXPropertyList propList;
+	librevenge::RVNGPropertyList propList;
 	switch (m_pageNumberPosition)
 	{
 	case TopLeft:
@@ -342,19 +373,28 @@ void WPSPageSpan::_insertPageNumberParagraph(WPXDocumentInterface *documentInter
 	default:
 		propList.insert("fo:text-align", "center");
 		break;
+	case None:
+	case TopLeftAndRight:
+	case TopInsideLeftAndRight:
+	case BottomLeftAndRight:
+	case BottomInsideLeftAndRight:
+		WPS_DEBUG_MSG(("WPSPageSpan::_insertPageNumberParagraph: unexpected value\n"));
+		propList.insert("fo:text-align", "center");
+		break;
 	}
 
-	documentInterface->openParagraph(propList, WPXPropertyListVector());
+	documentInterface->openParagraph(propList);
 
 	propList.clear();
 	propList.insert("style:font-name", m_pageNumberingFontName.cstr());
-	propList.insert("fo:font-size", m_pageNumberingFontSize, WPX_POINT);
+	propList.insert("fo:font-size", m_pageNumberingFontSize, librevenge::RVNG_POINT);
 	documentInterface->openSpan(propList);
 
 
 	propList.clear();
 	propList.insert("style:num-format", libwps::numberingTypeToString(m_pageNumberingType).c_str());
-	documentInterface->insertField("text:page-number", propList);
+	propList.insert("librevenge:field-format", "text:page-number");
+	documentInterface->insertField(propList);
 
 	propList.clear();
 	documentInterface->closeSpan();
@@ -363,34 +403,34 @@ void WPSPageSpan::_insertPageNumberParagraph(WPXDocumentInterface *documentInter
 }
 
 // -------------- manage header footer list ------------------
-void WPSPageSpan::_setHeaderFooter(HeaderFooterType type, HeaderFooterOccurence occurence, WPSSubDocumentPtr &doc)
+void WPSPageSpan::_setHeaderFooter(HeaderFooterType type, HeaderFooterOccurrence occurrence, WPSSubDocumentPtr &doc)
 {
-	if (occurence == NEVER) return;
+	if (occurrence == NEVER) return;
 
-	int pos = _getHeaderFooterPosition(type, occurence);
+	int pos = _getHeaderFooterPosition(type, occurrence);
 	if (pos == -1) return;
-	m_headerFooterList[pos]=WPSPageSpanInternal::HeaderFooterPtr(new WPSPageSpanInternal::HeaderFooter(type, occurence, doc));
+	m_headerFooterList[size_t(pos)]=WPSPageSpanInternal::HeaderFooterPtr(new WPSPageSpanInternal::HeaderFooter(type, occurrence, doc));
 }
 
-void WPSPageSpan::_removeHeaderFooter(HeaderFooterType type, HeaderFooterOccurence occurence)
+void WPSPageSpan::_removeHeaderFooter(HeaderFooterType type, HeaderFooterOccurrence occurrence)
 {
-	int pos = _getHeaderFooterPosition(type, occurence);
+	int pos = _getHeaderFooterPosition(type, occurrence);
 	if (pos == -1) return;
-	m_headerFooterList[pos].reset();
+	m_headerFooterList[size_t(pos)].reset();
 }
 
-bool WPSPageSpan::_containsHeaderFooter(HeaderFooterType type, HeaderFooterOccurence occurence)
+bool WPSPageSpan::_containsHeaderFooter(HeaderFooterType type, HeaderFooterOccurrence occurrence)
 {
-	int pos = _getHeaderFooterPosition(type, occurence);
-	if (pos == -1 || ! m_headerFooterList[pos]) return false;
-	if (!m_headerFooterList[pos]->getSubDocument()) return false;
+	int pos = _getHeaderFooterPosition(type, occurrence);
+	if (pos == -1 || ! m_headerFooterList[size_t(pos)]) return false;
+	if (!m_headerFooterList[size_t(pos)]->getSubDocument()) return false;
 	return true;
 }
 
-int WPSPageSpan::_getHeaderFooterPosition(HeaderFooterType type, HeaderFooterOccurence occurence)
+int WPSPageSpan::_getHeaderFooterPosition(HeaderFooterType type, HeaderFooterOccurrence occurrence)
 {
-	int typePos = 0, occurencePos = 0;
-	switch(type)
+	int typePos = 0, occurrencePos = 0;
+	switch (type)
 	{
 	case HEADER:
 		typePos = 0;
@@ -402,24 +442,28 @@ int WPSPageSpan::_getHeaderFooterPosition(HeaderFooterType type, HeaderFooterOcc
 		WPS_DEBUG_MSG(("WPSPageSpan::getVectorPosition: unknown type\n"));
 		return -1;
 	}
-	switch(occurence)
+	switch (occurrence)
 	{
 	case ALL:
-		occurencePos = 0;
+		occurrencePos = 0;
 		break;
 	case ODD:
-		occurencePos = 1;
+		occurrencePos = 1;
 		break;
 	case EVEN:
-		occurencePos = 2;
+		occurrencePos = 2;
 		break;
+	case FIRST:
+		occurrencePos = 3;
+		break;
+	case NEVER:
 	default:
-		WPS_DEBUG_MSG(("WPSPageSpan::getVectorPosition: unknown occurence\n"));
+		WPS_DEBUG_MSG(("WPSPageSpan::getVectorPosition: unknown occurrence\n"));
 		return -1;
 	}
-	int res = typePos*3+occurencePos;
+	int res = typePos*4+occurrencePos;
 	if (res >= int(m_headerFooterList.size()))
-		m_headerFooterList.resize(res+1);
+		m_headerFooterList.resize(size_t(res+1));
 	return res;
 }
 
